package javax.swing.plaf.metal;

import java.awt.event.MouseEvent;
import javax.swing.plaf.basic.BasicComboPopup;
import java.awt.Color;
import javax.swing.JList;
import javax.swing.JComboBox;
import java.awt.Dimension;
import java.awt.Container;
import java.awt.LayoutManager;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JButton;
import javax.swing.plaf.basic.ComboPopup;
import javax.swing.ComboBoxEditor;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Graphics;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.plaf.basic.BasicComboBoxUI;

public class MetalComboBoxUI extends BasicComboBoxUI
{
    public MetalComboBoxUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new MetalComboBoxUI();
    }
    
    @Override
    public void paint(Graphics g, JComponent c) {
        if (MetalLookAndFeel.usingOcean()) {
            super.paint(g, c);
        }
    }
    
    @Override
    public void paintCurrentValue(Graphics g, Rectangle bounds, boolean hasFocus) {
        if (MetalLookAndFeel.usingOcean()) {
            bounds.x += 2;
            bounds.width -= 3;
            if (this.arrowButton != null) {
                Insets insets = this.arrowButton.getInsets();
                bounds.y += insets.top;
                bounds.height -= insets.top + insets.bottom;
            }
            else {
                bounds.y += 2;
                bounds.height -= 4;
            }
            super.paintCurrentValue(g, bounds, hasFocus);
        }
        else if (g == null || bounds == null) {
            throw new NullPointerException("Must supply a non-null Graphics and Rectangle");
        }
    }
    
    @Override
    public void paintCurrentValueBackground(Graphics graphics, Rectangle rectangle, boolean b) {
        if (MetalLookAndFeel.usingOcean()) {
            graphics.setColor(MetalLookAndFeel.getControlDarkShadow());
            graphics.drawRect(rectangle.x, rectangle.y, rectangle.width, rectangle.height - 1);
            graphics.setColor(MetalLookAndFeel.getControlShadow());
            graphics.drawRect(rectangle.x + 1, rectangle.y + 1, rectangle.width - 2, rectangle.height - 3);
            if (b && !this.isPopupVisible(this.comboBox) && this.arrowButton != null) {
                graphics.setColor(this.listBox.getSelectionBackground());
                Insets insets = this.arrowButton.getInsets();
                if (insets.top > 2) {
                    graphics.fillRect(rectangle.x + 2, rectangle.y + 2, rectangle.width - 3, insets.top - 2);
                }
                if (insets.bottom > 2) {
                    graphics.fillRect(rectangle.x + 2, rectangle.y + rectangle.height - insets.bottom, rectangle.width - 3, insets.bottom - 2);
                }
            }
        }
        else if (graphics == null || rectangle == null) {
            throw new NullPointerException("Must supply a non-null Graphics and Rectangle");
        }
    }
    
    @Override
    public int getBaseline(JComponent component, int n, int n2) {
        int n3;
        if (MetalLookAndFeel.usingOcean() && n2 >= 4) {
            n2 -= 4;
            n3 = super.getBaseline(component, n, n2);
            if (n3 >= 0) {
                n3 += 2;
            }
        }
        else {
            n3 = super.getBaseline(component, n, n2);
        }
        return n3;
    }
    
    @Override
    protected ComboBoxEditor createEditor() {
        return new MetalComboBoxEditor.UIResource();
    }
    
    @Override
    protected ComboPopup createPopup() {
        return super.createPopup();
    }
    
    @Override
    protected JButton createArrowButton() {
        MetalComboBoxButton metalComboBoxButton = new MetalComboBoxButton(this.comboBox, new MetalComboBoxIcon(), this.comboBox.isEditable() || MetalLookAndFeel.usingOcean(), this.currentValuePane, this.listBox);
        metalComboBoxButton.setMargin(new Insets(0, 1, 1, 3));
        if (MetalLookAndFeel.usingOcean()) {
            metalComboBoxButton.putClientProperty(MetalBorders.NO_BUTTON_ROLLOVER, Boolean.TRUE);
        }
        this.updateButtonForOcean(metalComboBoxButton);
        return metalComboBoxButton;
    }
    
    private void updateButtonForOcean(JButton button) {
        if (MetalLookAndFeel.usingOcean()) {
            button.setFocusPainted(this.comboBox.isEditable());
        }
    }
    
    public PropertyChangeListener createPropertyChangeListener() {
        return new MetalComboBoxUI.MetalPropertyChangeListener();
    }
    
    @Deprecated
    protected void editablePropertyChanged(PropertyChangeEvent propertyChangeEvent) {
    }
    
    @Override
    protected LayoutManager createLayoutManager() {
        return new MetalComboBoxUI.MetalComboBoxLayoutManager();
    }
    
    public void layoutComboBox(Container container, MetalComboBoxUI.MetalComboBoxLayoutManager metalComboBoxLayoutManager) {
        if (this.comboBox.isEditable() && !MetalLookAndFeel.usingOcean()) {
            metalComboBoxLayoutManager.superLayout(container);
            return;
        }
        if (this.arrowButton != null) {
            if (MetalLookAndFeel.usingOcean()) {
                Insets insets2 = this.comboBox.getInsets();
                int width = this.arrowButton.getMinimumSize().width;
                this.arrowButton.setBounds(MetalUtils.isLeftToRight(this.comboBox) ? (this.comboBox.getWidth() - insets2.right - width) : insets2.left, insets2.top, width, this.comboBox.getHeight() - insets2.top - insets2.bottom);
            }
            else {
                Insets insets = this.comboBox.getInsets();
                this.arrowButton.setBounds(insets.left, insets.top, this.comboBox.getWidth() - (insets.left + insets.right), this.comboBox.getHeight() - (insets.top + insets.bottom));
            }
        }
        if (this.editor != null && MetalLookAndFeel.usingOcean()) {
            this.editor.setBounds(this.rectangleForCurrentValue());
        }
        return;
    }
    
    @Deprecated
    protected void removeListeners() {
        if (this.propertyChangeListener != null) {
            this.comboBox.removePropertyChangeListener(this.propertyChangeListener);
        }
    }
    
    public void configureEditor() {
        super.configureEditor();
    }
    
    public void unconfigureEditor() {
        super.unconfigureEditor();
    }
    
    @Override
    public Dimension getMinimumSize(JComponent component) {
        if (!this.isMinimumSizeDirty) {
            return new Dimension(this.cachedMinimumSize);
        }
        Dimension dimension;
        if (!this.comboBox.isEditable() && this.arrowButton != null) {
            Insets insets = this.arrowButton.getInsets();
            Insets insets2 = this.comboBox.getInsets();
            dimension = this.getDisplaySize();
            dimension.width += insets2.left + insets2.right;
            dimension.width += insets.right;
            dimension.width += this.arrowButton.getMinimumSize().width;
            dimension.height += insets2.top + insets2.bottom;
            dimension.height += insets.top + insets.bottom;
        }
        else if (this.comboBox.isEditable() && this.arrowButton != null && this.editor != null) {
            dimension = super.getMinimumSize(component);
            Insets margin = this.arrowButton.getMargin();
            dimension.height += margin.top + margin.bottom;
            dimension.width += margin.left + margin.right;
        }
        else {
            dimension = super.getMinimumSize(component);
        }
        this.cachedMinimumSize.setSize(dimension.width, dimension.height);
        this.isMinimumSizeDirty = false;
        return new Dimension(this.cachedMinimumSize);
    }
    
    static void access$500(MetalComboBoxUI metalComboBoxUI, JButton button) {
        metalComboBoxUI.updateButtonForOcean(button);
    }
    
    public class MetalPropertyChangeListener extends BasicComboBoxUI.PropertyChangeHandler
    {
        @Override
        public void propertyChange(PropertyChangeEvent e) {
            super.propertyChange(e);
            String propertyName = e.getPropertyName();
            if (propertyName == "editable") {
                if ((MetalComboBoxUI.this.arrowButton instanceof MetalComboBoxButton)) {
                    ((MetalComboBoxButton)MetalComboBoxUI.this.arrowButton).setIconOnly(MetalComboBoxUI.this.comboBox.isEditable() || MetalLookAndFeel.usingOcean());
                }
                MetalComboBoxUI.this.comboBox.repaint();
                MetalComboBoxUI.access$500(MetalComboBoxUI.this, MetalComboBoxUI.this.arrowButton);
            }
            else if (propertyName == "background") {
                Color color2 = (Color)e.getNewValue();
                MetalComboBoxUI.this.arrowButton.setBackground(color2);
                MetalComboBoxUI.this.listBox.setBackground(color2);
            }
            else if (propertyName == "foreground") {
                Color color = (Color)e.getNewValue();
                MetalComboBoxUI.this.arrowButton.setForeground(color);
                MetalComboBoxUI.this.listBox.setForeground(color);
            }
        }
    }
    
    public class MetalComboBoxLayoutManager extends BasicComboBoxUI.ComboBoxLayoutManager
    {
        @Override
        public void layoutContainer(Container container) {
            MetalComboBoxUI.this.layoutComboBox(container, this);
        }
        
        public void superLayout(Container parent) {
            super.layoutContainer(parent);
        }
    }
    
    @Deprecated
    public class MetalComboPopup extends BasicComboPopup
    {
        public MetalComboPopup(JComboBox combo) {
            super(combo);
        }
        
        public void delegateFocus(MouseEvent e) {
            super.delegateFocus(e);
        }
    }
}
